import usb
import random

RQ_soil_moisture = 1
RQ_Temp_Sensor = 2
RQ_Light_Sensor = 3
RQ_relay = 4
RQ_moter = 5
RQ_pin = 6

####################################
def find_mcu_boards():
    '''
    Find all Practicum MCU boards attached to the machine, then return a list
    of USB device handles for all the boards

    >>> devices = find_mcu_boards()
    >>> first_board = McuBoard(devices[0])
    '''
    boards = [dev for bus in usb.busses()
                  for dev in bus.devices
                  if (dev.idVendor,dev.idProduct) == (0x16c0,0x05dc)]
    return boards

####################################
class McuBoard:
    '''
    Generic class for accessing Practicum MCU board via USB connection.
    '''

    ################################
    def __init__(self, dev):
        self.device = dev
        self.handle = dev.open()

    ################################
    def usb_write(self, request, data=[], index=0, value=0):
        '''
        Send data output to the USB device (i.e., MCU board)
           request: request number to appear as bRequest field on the USB device
           index: 16-bit value to appear as wIndex field on the USB device
           value: 16-bit value to appear as wValue field on the USB device
        '''
        reqType = usb.TYPE_VENDOR | usb.RECIP_DEVICE | usb.ENDPOINT_OUT
        self.handle.controlMsg(
                reqType, request, data, value=value, index=index)

    ################################
    def usb_read(self, request, length=1, index=0, value=0):
        '''
        Request data input from the USB device (i.e., MCU board)
           request: request number to appear as bRequest field on the USB device
           length: number of bytes to read from the USB device
           index: 16-bit value to appear as wIndex field on the USB device
           value: 16-bit value to appear as wValue field on the USB device

        If successful, the method returns a tuple of length specified
        containing data returned from the MCU board.
        '''
        reqType = usb.TYPE_VENDOR | usb.RECIP_DEVICE | usb.ENDPOINT_IN
        buf = self.handle.controlMsg(
                reqType, request, length, value=value, index=index)
        return buf


####################################
class PeriBoard:

    ################################
    def __init__(self, mcu):
        self.mcu = mcu

    ################################
    def get_soil_moisture(self):
        value = self.mcu.usb_read(request=RQ_soil_moisture,length=2)
        return ((value[0] + value[1]*256)/1023) * 100

    def get_temperature(self):
        try:
            value = self.mcu.usb_read(request=RQ_Temp_Sensor,length=2)
            integer =  ( value[0] // 16 ) + value[1] * 16
            floating = 1 / ( value[0] % 16 )
            return integer + floating
        except ZeroDivisionError:
            return None

    def get_Lumen(self):
        value = self.mcu.usb_read(request=RQ_Light_Sensor,length=2)
        return value[0] + (value[1] % 2) * 256 

    def set_relay(self,state):
        self.mcu.usb_write(request=RQ_relay,value=state)

    def set_moter(self,state):
        self.mcu.usb_write(request=RQ_moter,value=state)

    def get_pin(self,index):
        value = self.mcu.usb_read(request=RQ_pin,index=index)
        return value[0]
